from abc import ABC
from typing import Optional
from datetime import datetime
import pandas as pd

class RagaSchemaElement(ABC):
    def __init__(self):
        super().__init__()
        self.type: str
        self.model: Optional[str]
        self.ref_col_name: Optional[str]

class PredictionSchemaElement(RagaSchemaElement):
    def __init__(self):
        super().__init__()
        self.type = "imageName"
        self.model = ""
        self.ref_col_name = ""

class TimeOfCaptureSchemaElement(RagaSchemaElement):
    def __init__(self):
        super().__init__()
        self.type = "timestamp"
        self.model = ""
        self.ref_col_name = ""

class FeatureSchemaElement(RagaSchemaElement):
    def __init__(self):
        super().__init__()
        self.type = "feature"
        self.model = ""
        self.ref_col_name = ""
class AttributeSchemaElement(RagaSchemaElement):
    def __init__(self):
        super().__init__()
        self.type = "attribute"
        self.model = ""
        self.ref_col_name = ""

class InferenceSchemaElement(RagaSchemaElement):
    def __init__(self, model:str):
        if not isinstance(model, str) or not model: 
            raise ValueError("model is required and must be a non-empty string.")
        self.type = "inference"
        self.model = model
        self.ref_col_name = ""

class ImageEmbeddingSchemaElement(RagaSchemaElement):
    def __init__(self, model:str, ref_col_name:str):
        if not isinstance(model, str) or not model: 
            raise ValueError("model is required and must be a non-empty string.")
        self.type = "imageEmbedding"
        self.model = model
        self.ref_col_name = ref_col_name


class RoiEmbeddingSchemaElement(RagaSchemaElement):
    def __init__(self, model:str, ref_col_name:str):
        if not isinstance(model, str) or not model: 
            raise ValueError("model is required and must be a non-empty string.")
        self.type = "roiEmbedding"
        self.model = model
        self.ref_col_name = ref_col_name



class RagaSchema():
    def __init__(self):
        self.columns = list()

    def validation(self, column_name: str, ragaSchemaElement:RagaSchemaElement, data_frame:pd.DataFrame):
        if not isinstance(column_name, str) or not column_name: 
            raise ValueError("column_name is required and must be a non-empty string.")
        if not isinstance(ragaSchemaElement, RagaSchemaElement): 
            raise ValueError("ragaSchemaElement must be an instance of the RagaSchemaElement.")
        if not isinstance(data_frame, pd.DataFrame): 
            raise ValueError("data_frame must be an instance of the pd.DataFrame.")
        return True
     
    def add(self, column_name: str, ragaSchemaElement, data_frame:pd.DataFrame):
        self.validation(column_name, ragaSchemaElement, data_frame)
        column_list = data_frame.columns.to_list()
        if column_name in column_list:
            self.columns.append({"customerColumnName":column_name, "type":ragaSchemaElement.type, "modelName":ragaSchemaElement.model, "ref_col_name":ragaSchemaElement.ref_col_name})
        else:
            raise ValueError(f"Raga Schema Error: Column name `{column_name}` not found in provided Data Frame. {column_list}")

class StringElement():
    def __init__(self, value:str):
        self.value = value

    def get(self):
        return self.value
    
class FloatElement():
    def __init__(self, value:float):
        self.value = value

    def get(self):
        return self.value
    
class TimeStampElement():
    def __init__(self, date_time:datetime):
        self.date_time = date_time

    def get(self):
        return self.date_time
    
class AggregationLevelElement():
    def __init__(self):
        self.levels = []

    def add(self, level:StringElement):
        self.levels.append(level.get())

    def get(self):
        return self.levels
    
class ModelABTestTypeElement():
    def __init__(self, type:str):
        self.type = type
        if self.type not in ["labelled", "unlabelled"]:
            raise ValueError("Invalid value for 'type'. Must be one of: ['labelled', 'unlabelled'].")

    def get(self):
        return self.type    

       
class ModelABTestRules():
    def __init__(self):
        self.rules = []

    def add(self, metric:StringElement, IoU:FloatElement, _class:StringElement, threshold:FloatElement):
        self.rules.append({ "metric" : metric.get(), "iou": IoU.get(),  "class": _class.get(), "threshold":threshold.get() })

    def get(self):
        return self.rules
class ObjectDetection:
    def __init__(self, Id:Optional[str], Format:Optional[str], Confidence:Optional[float], ClassId:Optional[str] = None, ClassName:Optional[str]=None, BBox=None):
        self.Id = Id
        self.ClassId = ClassId
        self.ClassName = ClassName
        self.BBox = BBox
        self.Format = Format
        self.Confidence = Confidence

class ImageDetectionObject():
    def __init__(self):
        self.detections = list()
    
    def add(self, object_detection:ObjectDetection):
        self.detections.append(object_detection.__dict__)
    
    def get(self):
        return self.__dict__

class Embedding:
    def __init__(self, embedding: float):
        self.embedding = embedding

class ImageEmbedding:
    def __init__(self):
         self.embeddings = []

    def add(self, embedding_values: Embedding):
        self.embeddings.append(embedding_values.embedding)

    def get(self):
        return self.__dict__

class ROIEmbedding:
    def __init__(self):
         self.embeddings = []

    def add(self, embedding_values: Embedding):
        self.embeddings.append(embedding_values.embedding)

    def get(self):
        return self.__dict__



from typing import Optional
from raga import Dataset, StringElement, AggregationLevelElement, ModelABTestRules, ModelABTestTypeElement, TestSession

def model_ab_test(test_session:TestSession, dataset_name: str, testName: StringElement, modelA: StringElement, modelB: StringElement,
                  type: ModelABTestTypeElement, aggregation_level: AggregationLevelElement, rules: ModelABTestRules,
                  gt: Optional[StringElement] = StringElement(""), filter: Optional[StringElement] = StringElement("")):
    dataset_id = ab_test_validation(test_session, dataset_name, testName, modelA, modelB, type, aggregation_level, rules, gt)      
    return {
            "datasetId": dataset_id,
            "experimentId": test_session.experiment_id,
            "name": testName.get(),
            "filter": filter.get(),
            "modelA": modelA.get(),
            "modelB": modelB.get(),
            "gt": gt.get(),
            "type": type.get(),
            "aggregationLevels": aggregation_level.get(),
            "rules": rules.get()
        }


def ab_test_validation(test_session:TestSession, dataset_name: str, testName: StringElement, modelA: StringElement, modelB: StringElement,
               type: ModelABTestTypeElement, aggregation_level: AggregationLevelElement, rules: ModelABTestRules,
               gt: Optional[StringElement] = StringElement("")):
    res_data = test_session.http_client.get(f"api/dataset?projectId={test_session.project_id}&name={dataset_name}", headers={"Authorization": f'Bearer {test_session.token}'})
    if not isinstance(res_data, dict):
            raise ValueError("Invalid response data. Expected a dictionary.")
    dataset_id = res_data.get("data", {}).get("id")
    if not dataset_id:
        raise KeyError("Invalid response data. Token not found.")
    
    assert isinstance(test_session, TestSession), "test_session must be an instance of the TestSession class."
    assert isinstance(dataset_name, str) and dataset_name, "dataset_name is required and must be str."
    assert isinstance(testName, StringElement) and testName.get(), "testName is required and must be an instance of the StringElement class."
    assert isinstance(modelA, StringElement) and modelA.get(), "modelA is required and must be an instance of the StringElement class."
    assert isinstance(modelB, StringElement) and modelB.get(), "modelB is required and must be an instance of the StringElement class."
    assert isinstance(type, ModelABTestTypeElement), "type must be an instance of the ModelABTestTypeElement class."
    assert isinstance(aggregation_level, AggregationLevelElement) and aggregation_level.get(), "aggregation_level is required and must be an instance of the AggregationLevelElement class."
    assert isinstance(rules, ModelABTestRules) and rules.get(), "rules is required and must be an instance of the ModelABTestRules class."

    if type.get() == "labelled":
        assert isinstance(gt, StringElement) and gt.get(), "gt is required on labelled type and must be an instance of the StringElement class."

    if type.get() == "unlabelled":
        if isinstance(gt, StringElement) and gt.get():
            raise ValueError("gt is not required on unlabelled type.")
    return dataset_id



"""Initialization file for the testing_platform package."""
import os
import logging

RAGA_CONFIG_FILE = ".raga/config"

# Get the value of the DEBUG environment variable
debug_mode = os.environ.get('DEBUG')

# Configure the logging format and level based on the DEBUG environment variable
if debug_mode:
    logging.basicConfig(
        format='%(asctime)s - %(name)s - %(levelname)s - %(message)s',
        level=logging.DEBUG
    )
else:
    logging.basicConfig(
        format='%(asctime)s - %(name)s - %(levelname)s - %(message)s',
        level=logging.INFO
    )
if debug_mode:
    # Add a file handler to log messages to a file
    file_handler = logging.FileHandler('debug.log')
    file_handler.setLevel(logging.DEBUG)
    file_handler.setFormatter(logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s'))

    # Add the file handler to the logger
    logger = logging.getLogger(__name__)
    logger.addHandler(file_handler)

from .test_session import *
from .dataset_creds import *
from .dataset import *
from .raga_schema import *
from ._tests import *



from raga import *
import pandas as pd
import json
import datetime

def get_timestamp_x_hours_ago(hours):
    current_time = datetime.datetime.now()
    delta = datetime.timedelta(days=90, hours=hours)
    past_time = current_time - delta
    timestamp = int(past_time.timestamp())
    return timestamp

ds_json_file = "improved_rtl.json"

test_df = []
with open(ds_json_file, 'r') as json_file:
    # Load JSON data
    json_data = json.load(json_file)
    hr = 1
    # Process the JSON data
    transformed_data = []
    for item in json_data:
        AnnotationsV1 = ImageDetectionObject()
        ROIVectorsM1 = ROIEmbedding()
        ImageVectorsM1 = ImageEmbedding()
        for detection in item["outputs"][0]["detections"]:
            AnnotationsV1.add(ObjectDetection(Id=0, ClassId=0, ClassName=detection['class'], Confidence=detection['confidence'], BBox= detection['bbox'], Format="xywh_normalized"))
            for roi_emb in detection['roi_embedding']:
                ROIVectorsM1.add(Embedding(roi_emb))
        
        attributes_dict = {}
        attributes = item.get("attributes", {})
        for key, value in attributes.items():
            attributes_dict[key] = StringElement(value)

        image_embeddings = item.get("image_embedding", {})
        for value in image_embeddings:
            ImageVectorsM1.add(Embedding(value))

        data_point = {
            'ImageId': StringElement(item["inputs"][0]),
            'TimeOfCapture': TimeStampElement(get_timestamp_x_hours_ago(hr)),
            'SourceLink': StringElement(item["inputs"][0]),
            'AnnotationsV1': AnnotationsV1,
            'ROIVectorsM1': ROIVectorsM1,
            'ImageVectorsM1': ImageVectorsM1,
        }

        merged_dict = {**data_point, **attributes_dict}
        test_df.append(merged_dict)
        hr+=1
        

pd_ds = pd.DataFrame(test_df)


# data_frame_extractor(pd_ds).to_csv("improved_rtl.csv", index=False)

schema = RagaSchema()
schema.add("ImageId", PredictionSchemaElement(), pd_ds)
schema.add("TimeOfCapture", TimeOfCaptureSchemaElement(), pd_ds)
schema.add("SourceLink", FeatureSchemaElement(), pd_ds)
schema.add("Reflection", AttributeSchemaElement(), pd_ds)
schema.add("Overlap", AttributeSchemaElement(), pd_ds)
schema.add("CameraAngle", AttributeSchemaElement(), pd_ds)
schema.add("AnnotationsV1", InferenceSchemaElement(model="57_improved"), pd_ds)
schema.add("ImageVectorsM1", ImageEmbeddingSchemaElement(model="57_improved", ref_col_name=""), pd_ds)
schema.add("ROIVectorsM1", RoiEmbeddingSchemaElement(model="57_improved", ref_col_name=""), pd_ds)

test_session = TestSession(project_name="testingProject",run_name= "test_iteration-test-90")

# Create an instance of the Dataset class
test_ds = Dataset(test_session=test_session, name="self_serve-Image-Editing")

test_ds.load(pd_ds, schema)




# testName = StringElement("testABNewTest-B-11")
# modelA = StringElement("57_improved")
# modelB = StringElement("57_improved")
# gt = StringElement("GT")
# type = ModelABTestTypeElement("labelled")
# aggregation_level = AggregationLevelElement()
# aggregation_level.add(StringElement("Reflection"))
# aggregation_level.add(StringElement("Overlap"))
# aggregation_level.add(StringElement("CameraAngle"))
# rules = ModelABTestRules()
# rules.add(metric = StringElement("precision_diff"), IoU = FloatElement(0.5), _class = StringElement("all"), threshold = FloatElement(0.05))

# model_comparison_check = model_ab_test(test_ds, testName=testName, modelA = modelA , modelB = modelB , gt = gt,  type = type, aggregation_level = aggregation_level, rules = rules)

# test_session.add(model_comparison_check)


testName = StringElement("testABNewTest-100")
modelA = StringElement("modelA")
modelB = StringElement("modelB")
type = ModelABTestTypeElement("unlabelled")
aggregation_level = AggregationLevelElement()
aggregation_level.add(StringElement("Reflection"))
aggregation_level.add(StringElement("Overlap"))
aggregation_level.add(StringElement("CameraAngle"))
rules = ModelABTestRules()
rules.add(metric = StringElement("difference_percentage"), IoU = FloatElement(0.5), _class = StringElement("all"), threshold = FloatElement(0.2))
rules.add(metric = StringElement("difference_count"), IoU = FloatElement(0.5), _class = StringElement("vehicle"), threshold = FloatElement(0.5))


model_comparison_check = model_ab_test(test_ds, testName=testName, modelA = modelA , modelB = modelB , type = type, aggregation_level = aggregation_level, rules = rules)

test_session.add(model_comparison_check)

test_session.run()


from raga.validators.dataset_creds_validations import DatasetCredsValidator
from raga.utils import HTTPClient

class DatasetCreds:
    def __init__(self, type: str, location: str, reference_id: str):
        self.type = DatasetCredsValidator.validate_type(type)
        self.location = DatasetCredsValidator.validate_location(location)
        self.reference_id = DatasetCredsValidator.validate_reference_id(reference_id)
        self.http_client = HTTPClient("https://api.example.com")


import json
import sys
import time
import requests
from raga.validators.test_session_validation import TestSessionValidator
from raga.utils import HTTPClient
from raga.utils import read_raga_config, get_config_value

class TestSession():
    MAX_RETRIES = 3
    RETRY_DELAY = 1
    ACCESS_KEY = "raga_access_key_id"
    SECRET_KEY = "raga_secret_access_key"


    def __init__(self, project_name: str, run_name: str, u_test=False):
        config_data = read_raga_config()
        self.project_name = TestSessionValidator.validate_project_name(project_name)
        self.run_name = TestSessionValidator.validate_run_name(run_name)
        self.api_host = get_config_value(config_data, 'default', 'api_host')
        self.raga_access_key_id = get_config_value(config_data, 'default', self.ACCESS_KEY)
        self.raga_secret_access_key = get_config_value(config_data, 'default', self.SECRET_KEY)
        self.http_client = HTTPClient(self.api_host)
        self.test_list = []
        self.added = False

        self.token = None
        self.project_id = None
        self.experiment_id = None
        if not u_test:
            self.initialize()


    def initialize(self):
        retries = 0
        while retries < self.MAX_RETRIES:
            try:
                self.token = self.create_token()
                self.project_id = self.get_project_id()
                self.experiment_id = self.create_experiment()
                break  # Exit the loop if initialization succeeds
            except requests.exceptions.RequestException as exception:
                print(f"Network error occurred: {str(exception)}")
                retries += 1
                if retries < self.MAX_RETRIES:
                    print(f"Retrying in {self.RETRY_DELAY} second(s)...")
                    time.sleep(self.RETRY_DELAY)
            except KeyError as exception:
                print(f"Key error occurred: {str(exception)}")
                sys.exit() # No need to retry if a KeyError occurs
            except ValueError as exception:
                print(f"Value error occurred: {str(exception)}")
                sys.exit() # No need to retry if a ValueError occurs
            except Exception as exception:
                print(f"An unexpected error occurred: {str(exception)}")
                sys.exit()  # No need to retry if an unexpected error occurs

    def add(self, model_comparison_check_payload):
        if not isinstance(model_comparison_check_payload, dict) or not model_comparison_check_payload:
            raise ValueError("model_comparison_check_payload must be a non-empty dictionary.")
        if model_comparison_check_payload.get("type") == "unlabelled":
            model_comparison_check_payload["api_end_point"] =  "api/experiment/test/unlabelled"
        if model_comparison_check_payload.get("type") == "labelled":
            model_comparison_check_payload["api_end_point"] = "api/experiment/test/labelled"
        self.test_list.append(model_comparison_check_payload)
        self.added = True

    def run(self):
        # Check if already added
        if not self.added:
            raise ValueError("add() is not called. Call add() before run().")
        if not len(self.test_list):
            raise ValueError("Test not found.")
        retries = 0
        while retries < self.MAX_RETRIES:
            try:
                for test_payload in self.test_list:
                    api_end_point = test_payload.get("api_end_point")
                    test_payload.pop("api_end_point")
                    res_data = self.http_client.post(api_end_point, data=test_payload, headers={"Authorization": f'Bearer {self.token}'})
                    if not isinstance(res_data, dict):
                        raise ValueError("Invalid response data. Expected a dictionary.")
                    print(res_data.get('data', ''))
                break  # Exit the loop if initialization succeeds
            except requests.exceptions.RequestException as exception:
                print(f"Network error occurred: {str(exception)}")
                retries += 1
                if retries < self.MAX_RETRIES:
                    print(f"Retrying in {self.RETRY_DELAY} second(s)...")
                    time.sleep(self.RETRY_DELAY)
            except KeyError as exception:
                print(f"Key error occurred: {str(exception)}")
                sys.exit() # No need to retry if a KeyError occurs
            except ValueError as exception:
                print(f"Value error occurred: {str(exception)}")
                sys.exit() # No need to retry if a ValueError occurs
            except Exception as exception:
                print(f"An unexpected error occurred: {str(exception)}")
                sys.exit()  # No need to retry if an unexpected error occurs

    def create_token(self):
        """
        Creates an authentication token by sending a request to the Raga API.

        Returns:
            str: The authentication token.

        Raises:
            KeyError: If the response data does not contain a valid token.
        """
        res_data = self.http_client.post(
            "api/token",
            {"accessKey": self.raga_access_key_id, "secretKey": self.raga_secret_access_key},
        )
        if not isinstance(res_data, dict):
            raise ValueError("Invalid response data. Expected a dictionary.")
        token = res_data.get("data", {}).get("token")
        if not token:
            raise KeyError("Invalid response data. Token not found.")
        return token


    def get_project_id(self):
        """
        Get project id by sending a request to the Raga API.

        Returns:
            str: The ID of the project.

        Raises:
            KeyError: If the response data does not contain a valid project ID.
            ValueError: If the response data is not in the expected format.
        """
        res_data = self.http_client.get(
            "api/project",
            params={"name": self.project_name},
            headers={"Authorization": f'Bearer {self.token}'},
        )

        if not isinstance(res_data, dict):
            raise ValueError("Invalid response data. Expected a dictionary.")

        project_id = res_data.get("data", {}).get("id")

        if not project_id:
            raise KeyError("Invalid response data. project ID not found.")
        return project_id



    def create_experiment(self):
        """
        Creates an experiment by sending a request to the Raga API.

        Returns:
            str: The ID of the created experiment.

        Raises:
            KeyError: If the response data does not contain a valid experiment ID.
            ValueError: If the response data is not in the expected format.
        """
        res_data = self.http_client.post(
            "api/experiment",
            {"name": self.run_name, "projectId": self.project_id},
            {"Authorization": f'Bearer {self.token}'},
        )

        if not isinstance(res_data, dict):
            raise ValueError("Invalid response data. Expected a dictionary.")

        experiment_id = res_data.get("data", {}).get("id")

        if experiment_id is None:
            raise KeyError("Invalid response data. Experiment ID not found.")
        return experiment_id



import json
import os
import sys
import time
import requests
from typing import Optional
import pandas as pd
from raga.validators.dataset_validations import DatasetValidator
from raga.dataset_creds import DatasetCreds
from raga import TestSession
import logging
import zipfile
from raga.utils import get_file_name, delete_files, upload_file, create_csv_and_zip_from_data_frame, data_frame_extractor, make_arg, on_upload_success, on_upload_failed

logger = logging.getLogger(__name__)
class Dataset():
    MAX_RETRIES = 3
    RETRY_DELAY = 1

    def __init__(
        self,
        test_session:TestSession,
        name: str,
        creds: Optional[DatasetCreds] = None,
        u_test = False
    ):
        self.test_session = test_session
        self.name = DatasetValidator.validate_name(name)
        self.creds = DatasetValidator.validate_creds(creds)
        self.csv_file = f"experiment_{test_session.experiment_id}.csv"
        self.zip_file = f"experiment_{test_session.experiment_id}.zip"
        self.dataset_id = None
        if not u_test:
            self.dataset_id = self.create_dataset()
        self.dataset_file_id = None
        self.data_set_top_five_rows = {}


    def initialize(
        self,
        data = None, 
        schema = None, 
        format= None,
        model_name = None,
        inference_col_name= None,
        embedding_col_name= None):
        from raga import RagaSchema
        # Validate and process the data argument
        if data is not None:
            if isinstance(data, str):
                # Assume it's a file path
                self.load_labels_from_file(
                data,
                format,
                model_name,
                inference_col_name,
                embedding_col_name
            )
            elif isinstance(data, pd.DataFrame):

                if not data.empty:
                    # Validate and process the schema argument
                    if schema is not None:
                        if isinstance(schema, RagaSchema):
                            # Use the provided schema object
                            dataset_column = schema.columns
                            # Use the provided DataFrame
                            self.load_data_frame(data_frame_extractor(data), dataset_column)
                            # print(json.dumps(dataset_column))

                        else:
                            raise ValueError("Invalid schema argument. Expected an instance of RagaSchema.")
                else:
                    raise ValueError("Empty DataFrame data argument.")
            else:
                raise ValueError("Invalid data argument. Expected a DataFrame or a file path.")
    
    def load(
        self,
        data = None, 
        schema = None, 
        format= None,
        model_name = None,
        inference_col_name= None,
        embedding_col_name= None
    ):
        retries = 0
        while retries < self.MAX_RETRIES:
            try:
                self.initialize(data, schema, format, model_name, inference_col_name, embedding_col_name)
                print("Data loaded successful!")
                break  # Exit the loop if initialization succeeds
            except requests.exceptions.RequestException as e:
                print(f"Network error occurred: {str(e)}")
                retries += 1
                if retries < self.MAX_RETRIES:
                    print(f"Retrying in {self.RETRY_DELAY} second(s)...")
                    time.sleep(self.RETRY_DELAY)
            except KeyError as e:
                print(f"Key error occurred: {str(e)}")
                sys.exit()# No need to retry if a KeyError occurs 
            except ValueError as e:
                print(f"Value error occurred: {str(e)}")
                sys.exit() # No need to retry if a ValueError occurs
            except Exception as e:
                print(f"An unexpected error occurred: {str(e)}")
                sys.exit() # No need to retry if an unexpected error occurs


    def load_data_frame(self, data_frame:pd.DataFrame, dataset_column):
        """
        Loads the data frame, creates a CSV file, zips it, and uploads it to the server.
        """
        logger.debug("Loading Data Frames")
        create_csv_and_zip_from_data_frame(data_frame, self.csv_file, self.zip_file)
        signedUploadPath, filePath = self.get_pre_signed_s3_url(self.zip_file)

        upload_file(
            signedUploadPath,
            self.zip_file,
            success_callback=on_upload_success,
            failure_callback=on_upload_failed,
        )
        delete_files(self.csv_file, self.zip_file)

        self.dataset_file_id = self.create_dataset_load_definition(filePath, "csv", dataset_column)
        self.notify_server()

    def load_labels_from_file(
        self,
        path_to_file,
        format,
        model_name,
        inference_col_name,
        embedding_col_name
    ):
        """
        Loads labels from a file, zips it, and uploads it to the server.

        Args:
            path_to_file (str): The path to the file containing labels.
            format (str): The format of the labels.
            model_name (str): The name of the model.
            inference_col_name (str): The name of the inference column.
            embedding_col_name (str): The name of the embedding column.

        Raises:
            ValueError: If any required parameter is missing.
        """
        required_params = [
            "format",
            "model_name",
            "inference_col_name",
            "embedding_col_name",
        ]
        for param in required_params:
            if not locals().get(param):
                raise ValueError(
                    f"{param.capitalize().replace('_', ' ')} is required.")
        file_dir = os.path.dirname(path_to_file)
        self.column_validate(path_to_file, format, inference_col_name, embedding_col_name)
        file_name_without_ext, file_extension, file_name = get_file_name(
            path_to_file)
        zip_file_name = os.path.join(file_dir, file_name_without_ext + ".zip")
        with zipfile.ZipFile(zip_file_name, "w") as zip_file:
            zip_file.write(path_to_file, file_name)
        signedUploadPath, filePath  = self.get_pre_signed_s3_url(
            file_name_without_ext + ".zip")
        upload_file(
            signedUploadPath,
            zip_file_name,
            success_callback=on_upload_success,
            failure_callback=on_upload_failed,
        )
        if os.path.exists(zip_file_name):
            os.remove(zip_file_name)
            logger.debug("Zip file deleted")
        else:
            logger.debug("Zip file not found")

        arguments = make_arg(model_name, inference_col_name, embedding_col_name)

        self.dataset_file_id = self.create_dataset_load_definition(filePath, format, arguments)
        self.notify_server()
    
    def column_validate(self, file_dir, format, inference_col_name, embedding_col_name):
        if format == 'coco':
            with open(file_dir, 'r') as f:
                json_data = json.load(f)
            if json_data.get(inference_col_name) is None:
                raise ValueError(f"Raga Schema Error: Column name `{inference_col_name}` not found in provided coco file. {list(json_data.keys())}")
            if json_data.get(embedding_col_name) is None:
                raise ValueError(f"Raga Schema Error: Column name `{embedding_col_name}` not found in provided coco file. {list(json_data.keys())}")
        return True

    def head(self):
        res_data = self.test_session.http_client.get(
            f"api/dataset/{self.dataset_id}/data",
            headers={"Authorization": f'Bearer {self.test_session.token}'},
        )
        if not res_data or 'data' not in res_data or 'rows' not in res_data['data'] or 'columns' not in res_data['data']:
            raise ValueError("Record not found!")
        
        print(self.filter_head(res_data.get('data', {}).get('rows', {}), res_data.get('data', {}).get('columns', {})))

    def filter_head(self, rows, columns):
        pd_data = pd.DataFrame(rows)
        columns_temp = []
        for col in columns:
            columns_temp.append(col.get("columnName"))
        return pd_data.get(columns_temp)
    
    def get_pre_signed_s3_url(self, file_name: str):
        """
        Generates a pre-signed URL for uploading the file to an S3 bucket.

        Args:
            file_name (str): The name of the file.

        Returns:
            str: The pre-signed S3 URL.

        Raises:
            ValueError: If the file name is missing.
        """
        res_data = self.test_session.http_client.get(
            "api/dataset/uploadpath",
            None,{"experimentId": self.test_session.experiment_id, "fileName": file_name, "contentType":"application/zip"},
            {"Authorization": f'Bearer {self.test_session.token}'},
        )
        logger.debug("Pre-signed URL generated")
        print("Pre-signed URL generated")
        return res_data["data"]["signedUploadPath"], res_data["data"]["filePath"]


    def notify_server(self):
        """
        Notifies the server to load the dataset with the provided experiment ID and data definition.
        """
        res_data = self.test_session.http_client.post(
            "api/experiment/load-data",
            {"experimentId": self.test_session.experiment_id, "datasetFileId": self.dataset_file_id},
            {"Authorization": f'Bearer {self.test_session.token}'},
        )
        print(res_data.get('data', ''))
        return res_data

    def create_dataset(self):
        if not self.test_session.project_id:
            raise ValueError("Project ID is required.")
        if not self.test_session.token:
            raise ValueError("Token is required.")

        res_data = self.test_session.http_client.post(
            "api/dataset",
            {"name": self.name, "projectId": self.test_session.project_id, "experimentId":self.test_session.experiment_id},
            {"Authorization": f'Bearer {self.test_session.token}'},
        )

        if not res_data or 'data' not in res_data or 'id' not in res_data['data']:
            raise ValueError("Failed to create dataset.")

        return res_data['data']['id']

    
    def create_dataset_load_definition(self, filePath: str, type: str, arguments: dict):
        payload = {
            "datasetId": self.dataset_id,
            "filePath": filePath,
            "type": type,
            "arguments": arguments
        }

        res_data = self.test_session.http_client.post(
            "api/dataset/definition", payload,
            {"Authorization": f'Bearer {self.test_session.token}'},
        )
        return res_data.get('data',{}).get('id')






import json
import logging
import requests
from urllib.parse import urlparse

logger = logging.getLogger(__name__)

class HTTPClient:
    def __init__(self, base_url: str):
        self.base_url = self.validate_base_url(base_url)
        logger.debug(f"Base URL: {self.base_url}")

    def validate_base_url(self, base_url: str) -> str:
        """
        Validates the base URL format and returns the validated URL.

        Args:
            base_url (str): The base URL to validate.

        Returns:
            str: The validated base URL.

        Raises:
            ValueError: If the base URL format is invalid.
        """
        parsed_url = urlparse(base_url)
        if not all([parsed_url.scheme, parsed_url.netloc]):
            raise ValueError("Invalid base URL. Must be in the format 'http(s)://domain.com'.")
        return base_url

    def get(self, endpoint: str, params=None, data=None, headers=None):
        """
        Sends a GET request to the specified endpoint.

        Args:
            endpoint (str): The endpoint to send the GET request to.
            params (dict, optional): The query parameters for the GET request. Defaults to None.
            data (dict, optional): The request payload for the GET request. Defaults to None.
            headers (dict, optional): The headers for the GET request. Defaults to None.

        Returns:
            dict: The JSON response from the GET request.

        Raises:
            ValueError: If the GET request is unsuccessful or returns an error response.
        """
        url = self.base_url + endpoint
        logger.debug(f"API ENDPOINT {endpoint}")
        logger.debug(f"API PARAMS {json.dumps(params)}")
        logger.debug(f"API DATA {json.dumps(data)}")
        logger.debug(f"API HEADER {json.dumps(headers)}")

        default_headers = {'Content-Type': 'application/json'}
        if headers:
            headers = {**default_headers, **headers}
        else:
            headers = default_headers
            
        if data:
            data = json.dumps(data)

        response = requests.get(url, params=params, data=data, headers=headers)
        logger.debug(f"API RESPONSE {response.json()}")
        status_code = response.status_code
        json_data = response.json()

        if status_code in (200, 201) and json_data.get("success"):
            print(json_data.get("message"))
            logger.debug(json_data.get("message"))
            return json_data
        else:
            error_message = json_data.get("message")
            if error_message:
                raise ValueError(f"Request failed with status code {status_code}: {error_message}")
            else:
                raise ValueError(f"Request failed with status code {status_code}")

    def post(self, endpoint: str, data=None, headers=None):
        """
        Sends a POST request to the specified endpoint.

        Args:
            endpoint (str): The endpoint to send the POST request to.
            data (dict, optional): The request payload for the POST request. Defaults to None.
            headers (dict, optional): The headers for the POST request. Defaults to None.

        Returns:
            dict: The JSON response from the POST request.

        Raises:
            ValueError: If the POST request is unsuccessful or returns an error response.
        """
        url = self.base_url + endpoint
        logger.debug(f"API ENDPOINT {endpoint}")
        logger.debug(f"API DATA {json.dumps(data)}")
        logger.debug(f"API HEADER {json.dumps(headers)}")

        default_headers = {'Content-Type': 'application/json'}
        if headers:
            headers = {**default_headers, **headers}
        else:
            headers = default_headers

        response = requests.request('POST', url, json=data, headers=headers)
        logger.debug(f"API RESPONSE {response.json()}")
        status_code = response.status_code
        json_data = response.json()

        if status_code in (200, 201) and json_data.get("success"):
            print(json_data.get("message"))
            logger.debug(json_data.get("message"))
            return json_data
        else:
            error_message = json_data.get("message")
            if error_message:
                raise ValueError(f"Request failed with status code {status_code}: {error_message}")
            else:
                raise ValueError(f"Request failed with status code {status_code}")





import os


def get_file_name(file_path):
    """
    Extracts the file name, extension, and file name without extension from the given file path.

    Args:
        file_path (str): The path of the file.

    Returns:
        Tuple[str, str, str]: The file name without extension, file extension, and file name.

    Raises:
        ValueError: If the file path is invalid.
    """
    if not isinstance(file_path, str):
        raise ValueError(
            "Invalid file path. Please provide a valid file path string.")

    file_name = os.path.basename(file_path)
    file_name_without_ext, file_extension = os.path.splitext(file_name)

    if file_extension:
        return file_name_without_ext, file_extension, file_name
    else:
        return file_name_without_ext, "", file_name


from raga import *
import pandas as pd
import json

ds_json_file = "test-dataset-modelG.json"

test_df = []
with open(ds_json_file, 'r') as json_file:
    # Load JSON data
    json_data = json.load(json_file)
    
    # Process the JSON data
    transformed_data = []
    for item in json_data:
        AnnotationsV1 = ImageDetectionObject()
        ROIVectorsM1 = ROIEmbedding()
        ImageVectorsM1 = ImageEmbedding()
        for detection in item["outputs"][0]["detections"]:
            AnnotationsV1.add(ObjectDetection(Id=0, ClassId=0, ClassName=detection['class'], Confidence=detection['confidence'], BBox= detection['bbox'], Format="xywh_normalized"))
            for roi_emb in detection['roi_embedding']:
                ROIVectorsM1.add(Embedding(roi_emb))
        
        attributes_dict = {}
        attributes = item.get("attributes", {})
        for key, value in attributes.items():
            attributes_dict[key] = StringElement(value)

        image_embeddings = item.get("image_embedding", {})
        for value in image_embeddings:
            ImageVectorsM1.add(Embedding(value))

        data_point = {
            'ImageId': StringElement(item["inputs"][0]),
            'TimeOfCapture': TimeStampElement(item["capture_time"]),
            'SourceLink': StringElement(item["inputs"][0]),
            'AnnotationsGroundTruth': AnnotationsV1,
            'ImageVectorsGroundTruth': ROIVectorsM1,
            'ROIVectorsGroundTruth': ImageVectorsM1,
        }

        merged_dict = {**data_point, **attributes_dict}

        test_df.append(merged_dict)
        

pd_data_frame = pd.DataFrame(test_df)

model_name = "Testing-groundTruth"

#create schema object of RagaSchema instance
schema = RagaSchema()
schema.add("ImageId", PredictionSchemaElement(), pd_data_frame)
schema.add("TimeOfCapture", TimeOfCaptureSchemaElement(), pd_data_frame)
schema.add("SourceLink", FeatureSchemaElement(), pd_data_frame)
schema.add("Reflection", AttributeSchemaElement(), pd_data_frame)
schema.add("Overlap", AttributeSchemaElement(), pd_data_frame)
schema.add("CameraAngle", AttributeSchemaElement(), pd_data_frame)
schema.add("AnnotationsGroundTruth", InferenceSchemaElement(model=model_name), pd_data_frame)
schema.add("ImageVectorsGroundTruth", ImageEmbeddingSchemaElement(model=model_name, ref_col_name=""), pd_data_frame)
schema.add("ROIVectorsGroundTruth", RoiEmbeddingSchemaElement(model=model_name, ref_col_name=""), pd_data_frame)

project_name = "testingProject" # Project Name
run_name= "test-exp-jun-new-18-2" # Experiment Name
dataset_name = "coco-test-dataset-1" # Dataset Name


#create test_session object of TestSession instance
test_session = TestSession(project_name=project_name,run_name=run_name, u_test=True)
test_session.project_id=1
test_session.experiment_id=795
test_session.token="eyJhbGciOiJIUzUxMiJ9.eyJzdWIiOiJhZG1pbkByYWdhIiwicm9sZXMiOlsiUk9MRV9BRE1JTiJdLCJ1c2VyTmFtZSI6ImFkbWluQHJhZ2EiLCJleHAiOjE2ODg3MjY0NDIsImlhdCI6MTY4ODY0MDA0Miwib3JnSWQiOjEsImp0aSI6ImFkbWluQHJhZ2EifQ.GoC2l4OW8F1ZnRZO1-R8D0qvrewHuJ8JyKkFNC1IX5jRXmZdT5gjSrYTfrdl7g2ShXZvAIOerOwKzVe8EDEKCA"
#create test_ds object of Dataset instance
test_ds = Dataset(test_session=test_session, name=dataset_name)

#load schema and pandas data frame
# test_ds.load(pd_data_frame, schema)
test_ds.load(data="/Users/manabroy/localhost/observance/raga/testing_platform/testing-platform-python-client/raga/examples/modelA_coco.json", format="coco", model_name="ModelA", inference_col_name="annotations", embedding_col_name="embeddings")
# test_ds.head()






# Params for unlabelled AB Model Testing

# testName = StringElement("test-labelled-jun-1")
# modelA = StringElement("Testing-modelA")
# modelB = StringElement("Testing-modelB")
# gt = StringElement("Testing-groundTruth")
# type = ModelABTestTypeElement("labelled")
# aggregation_level = AggregationLevelElement()
# aggregation_level.add(StringElement("Reflection"))
# aggregation_level.add(StringElement("Overlap"))
# aggregation_level.add(StringElement("CameraAngle"))
# rules = ModelABTestRules()
# rules.add(metric = StringElement("precision_diff"), IoU = FloatElement(0.5), _class = StringElement("ALL"), threshold = FloatElement(0.5))

# #create payload for model ab testing
# model_comparison_check = model_ab_test(test_session, dataset_name=dataset_name, testName=testName, modelA = modelA , modelB = modelB , type = type, aggregation_level = aggregation_level, rules = rules, gt=gt)


# #add payload into test_session object
# test_session.add(model_comparison_check)

# #run added ab test model payload
# test_session.run()


import unittest
from unittest import mock
from unittest.mock import MagicMock, patch, Mock, call
from requests import Response
import requests
from raga import TestSession

class TestTestSession(unittest.TestCase):

    def setUp(self):
        self.project_name = "project_name"
        self.run_name = "run_name"
        self.mock_http_client = MagicMock()
        self.test_session = TestSession(self.project_name, self.run_name, u_test=True)
        self.test_session.http_client = self.mock_http_client

    def test_create_token(self):
        expected_token = "token"
        mock_response_data = {"data": {"token": expected_token}}
        self.mock_http_client.post.return_value = mock_response_data

        # Call the method
        token = self.test_session.create_token()

        # Assert the expected token is returned
        self.assertEqual(token, expected_token)

    def test_create_token_missing_token(self):
        mock_response_data = {"data": {}}
        self.mock_http_client.post.return_value = mock_response_data

        # Call the method and assert it raises a KeyError
        with self.assertRaises(KeyError):
            self.test_session.create_token()

    def test_get_project_id(self):
        expected_project_id = "project_id"
        mock_response_data = {"data": {"id": expected_project_id}}
        self.mock_http_client.get.return_value = mock_response_data

        # Call the method
        project_id = self.test_session.get_project_id()

        # Assert the expected project ID is returned
        self.assertEqual(project_id, expected_project_id)

    def test_get_project_id_missing_project_id(self):
        mock_response_data = {"data": {}}
        self.mock_http_client.get.return_value = mock_response_data

        # Call the method and assert it raises a KeyError
        with self.assertRaises(KeyError):
            self.test_session.get_project_id()

    def test_create_experiment(self):
        expected_experiment_id = "experiment_id"
        mock_response_data = {"data": {"id": expected_experiment_id}}
        self.mock_http_client.post.return_value = mock_response_data

        # Call the method
        experiment_id = self.test_session.create_experiment()

        # Assert the expected experiment ID is returned
        self.assertEqual(experiment_id, expected_experiment_id)

    def test_create_experiment_missing_experiment_id(self):
        mock_response_data = {"data": {}}
        self.mock_http_client.post.return_value = mock_response_data

        # Call the method and assert it raises a KeyError
        with self.assertRaises(KeyError):
            self.test_session.create_experiment()

    @patch("raga.test_session.HTTPClient")
    def test_get_project_id(self, mock_http_client):
        # Create a TestSession instance
        test_session = TestSession("project_name", "run_name", u_test=True)

        # Mock the HTTPClient instance and its get method
        mock_get = Mock(return_value={"data": {"id": "project_id"}})
        mock_http_client.return_value.get = mock_get

        # Call the get_project_id method
        project_id = test_session.get_project_id()

        # Verify the expected method calls on the mock HTTPClient
        mock_http_client.assert_called_once()
        mock_get.assert_called_once_with(
            "api/project",
            params={"name": "project_name"},
            headers={"Authorization": f'Bearer {test_session.token}'},
        )

        # Verify the returned project_id
        self.assertEqual(project_id, "project_id")

    @patch("raga.test_session.HTTPClient")
    def test_get_project_id_no_data(self, mock_http_client):
        # Create a TestSession instance
        test_session = TestSession("project_name", "run_name", u_test=True)
        # Mock the HTTPClient instance and its get method
        mock_get = Mock(return_value={})
        mock_http_client.return_value.get = mock_get

        with self.assertRaises(KeyError):
            test_session.get_project_id()

        mock_http_client.assert_called_once()
        mock_get.assert_called_once_with(
            "api/project",
            params={"name": "project_name"},
            headers={"Authorization": f'Bearer {self.test_session.token}'},
        )

    @patch("raga.test_session.HTTPClient")
    def test_get_project_id_invalid_response(self, mock_http_client):
        # Create a TestSession instance
        test_session = TestSession("project_name", "run_name", u_test=True)
        # Mock the HTTPClient instance and its get method
        mock_get = Mock(return_value="invalid_response")
        mock_http_client.return_value.get = mock_get

        with self.assertRaises(ValueError):
            test_session.get_project_id()

        mock_http_client.assert_called_once()
        mock_get.assert_called_once_with(
            "api/project",
            params={"name": "project_name"},
            headers={"Authorization": f'Bearer {self.test_session.token}'},
        )

    @patch("raga.test_session.HTTPClient")
    def test_create_experiment(self, mock_http_client):
        # Create a TestSession instance
        test_session = TestSession("project_name", "run_name", u_test=True)
        # Mock the HTTPClient instance and its post method
        mock_post = Mock(return_value={"data": {"id": "experiment_id"}})
        mock_http_client.return_value.post = mock_post

        experiment_id = test_session.create_experiment()

        mock_http_client.assert_called_once()
        mock_post.assert_called_once_with(
            "api/experiment",
            {"name": "run_name", "projectId": self.test_session.project_id},
            {"Authorization": f'Bearer {self.test_session.token}'},
        )
        self.assertEqual(experiment_id, "experiment_id")

    @patch("raga.test_session.HTTPClient")
    def test_create_experiment_no_data(self, mock_http_client):
        # Create a TestSession instance
        test_session = TestSession("project_name", "run_name", u_test=True)
        # Mock the HTTPClient instance and its post method
        mock_post = Mock(return_value={})
        mock_http_client.return_value.post = mock_post

        with self.assertRaises(KeyError):
            test_session.create_experiment()

        mock_http_client.assert_called_once()
        mock_post.assert_called_once_with(
            "api/experiment",
            {"name": "run_name", "projectId": self.test_session.project_id},
            {"Authorization": f'Bearer {self.test_session.token}'},
        )

    @patch("raga.test_session.HTTPClient")
    def test_create_experiment_invalid_response(self, mock_http_client):
        # Create a TestSession instance
        test_session = TestSession("project_name", "run_name", u_test=True)
        # Mock the HTTPClient instance and its post method
        mock_post = Mock(return_value="invalid_response")
        mock_http_client.return_value.post = mock_post

        with self.assertRaises(ValueError):
            test_session.create_experiment()

        mock_http_client.assert_called_once()
        mock_post.assert_called_once_with(
            "api/experiment",
            {"name": "run_name", "projectId": self.test_session.project_id},
            {"Authorization": f'Bearer {self.test_session.token}'},
        )
    
    def test_initialize_successful(self):
        mock_create_token = MagicMock(return_value="token")
        mock_get_project_id = MagicMock(return_value="project_id")
        mock_create_experiment = MagicMock(return_value="experiment_id")

        self.test_session.create_token = mock_create_token
        self.test_session.get_project_id = mock_get_project_id
        self.test_session.create_experiment = mock_create_experiment

        # Call the method
        self.test_session.initialize()

        # Assert the expected values are set
        self.assertEqual(self.test_session.token, "token")
        self.assertEqual(self.test_session.project_id, "project_id")
        self.assertEqual(self.test_session.experiment_id, "experiment_id")
        mock_create_token.assert_called_once()
        mock_get_project_id.assert_called_once()
        mock_create_experiment.assert_called_once()

    def test_initialize_network_error(self):
        mock_create_token = MagicMock(side_effect=requests.exceptions.RequestException("Network error"))
        mock_get_project_id = MagicMock()
        mock_create_experiment = MagicMock()

        self.test_session.create_token = mock_create_token
        self.test_session.get_project_id = mock_get_project_id
        self.test_session.create_experiment = mock_create_experiment

        # Call the method
        with patch('builtins.print') as mock_print:
            # Set the MAX_RETRIES to 3 for testing
            self.test_session.MAX_RETRIES = 3
            self.test_session.initialize()

            # Assert the expected prints and retries
            expected_calls = [
                call('Network error occurred: Network error'),
                call(f'Retrying in {self.test_session.RETRY_DELAY} second(s)...'),
                call('Network error occurred: Network error'),
                call(f'Retrying in {self.test_session.RETRY_DELAY} second(s)...'),
                call('Network error occurred: Network error')
            ]
            mock_print.assert_has_calls(expected_calls)
            self.assertEqual(mock_print.call_count, len(expected_calls))
            self.assertEqual(mock_create_token.call_count, self.test_session.MAX_RETRIES)
            self.assertEqual(mock_get_project_id.call_count, 0)
            self.assertEqual(mock_create_experiment.call_count, 0)

    def test_initialize_key_error(self):
        mock_create_token = MagicMock(return_value="token")
        mock_get_project_id = MagicMock(side_effect=KeyError("Invalid response data"))
        mock_create_experiment = MagicMock()

        self.test_session.create_token = mock_create_token
        self.test_session.get_project_id = mock_get_project_id
        self.test_session.create_experiment = mock_create_experiment
       
        # Call the method
        with patch('builtins.print') as mock_print, \
            self.assertRaises(SystemExit) as cm:
            self.test_session.initialize()

            # Assert the expected prints and no retries
            expected_calls = [
                call("Key error occurred: 'Invalid response data'")
            ]
            mock_print.assert_has_calls(expected_calls, any_order=False)
            self.assertEqual(mock_print.call_count, 1)
            self.assertEqual(mock_create_token.call_count, 1)
            self.assertEqual(mock_get_project_id.call_count, 1)
            self.assertEqual(mock_create_experiment.call_count, 0)
            self.assertEqual(cm.exception.code, 1)
            
    
    def test_initialize_value_error(self):
        mock_create_token = MagicMock(return_value="token")
        mock_get_project_id = MagicMock(side_effect=ValueError("Invalid response data. Expected a dictionary."))
        mock_create_experiment = MagicMock()

        self.test_session.create_token = mock_create_token
        self.test_session.get_project_id = mock_get_project_id
        self.test_session.create_experiment = mock_create_experiment

        # Call the method
        with patch('builtins.print') as mock_print, \
            self.assertRaises(SystemExit) as cm:
            self.test_session.initialize()

            # Assert the expected prints and no retries
            expected_calls = [
                call('Value error occurred: Invalid response data. Expected a dictionary.')
            ]
            mock_print.assert_has_calls(expected_calls, any_order=False)
            self.assertEqual(mock_print.call_count, 1)
            self.assertEqual(mock_create_token.call_count, 1)
            self.assertEqual(mock_get_project_id.call_count, 1)
            self.assertEqual(mock_create_experiment.call_count, 0)
            self.assertEqual(cm.exception.code, 1)


    def test_initialize_unexpected_error(self):
        mock_create_token = MagicMock(return_value="token")
        mock_get_project_id = MagicMock(return_value="project_id")
        mock_create_experiment = MagicMock(side_effect=Exception("An unexpected error occurred"))

        self.test_session.create_token = mock_create_token
        self.test_session.get_project_id = mock_get_project_id
        self.test_session.create_experiment = mock_create_experiment
    
        # Call the method
        with patch('builtins.print') as mock_print, \
             self.assertRaises(SystemExit) as cm:
            self.test_session.initialize()

            # Assert the expected prints and no retries
            expected_calls = [
                call("An unexpected error occurred: An unexpected error occurred")
            ]
            mock_print.assert_has_calls(expected_calls, any_order=False)
            self.assertEqual(mock_print.call_count, 1)
            self.assertEqual(mock_create_token.call_count, 1)
            self.assertEqual(mock_get_project_id.call_count, 1)
            self.assertEqual(mock_create_experiment.call_count, 1)
            self.assertEqual(cm.exception.code, 1)

    def test_run_successful(self):
        payload = {"key": "value"}

        # Call the method
        self.test_session.add(payload)

        self.assertEqual(len(self.test_session.test_list), 1)
        self.assertTrue(self.test_session.added)
    
    def test_add_empty_payload(self):
        with self.assertRaises(ValueError) as cm:
            self.test_session.add({})
        self.assertEqual(str(cm.exception), "model_comparison_check_payload must be a non-empty dictionary.")
        self.assertEqual(len(self.test_session.test_list), 0)
        self.assertFalse(self.test_session.added)

    def test_run_success(self):
        self.test_session.added = True
        self.test_session.test_list = [{"key": "value"}]
        self.test_session.token = "test_token"
        
        mock_post = MagicMock(return_value={"data": "test_result"})
        with patch.object(self.test_session.http_client, 'post', mock_post):
            self.test_session.run()

        mock_post.assert_called_once_with("api/experiment/test", data={"key": "value"}, headers={"Authorization": "Bearer test_token"})

    def test_run_not_added(self):
        with self.assertRaises(ValueError) as cm:
            self.test_session.run()

        self.assertEqual(str(cm.exception), "add() is not called. Call add() before run().")

    def test_run_empty_list(self):
        self.test_session.added = True

        with self.assertRaises(ValueError) as cm:
            self.test_session.run()

        self.assertEqual(str(cm.exception), "Test not found.")

    def test_run_network_error_retry(self):
        self.test_session.added = True
        self.test_session.token = "test_token"
        self.test_session.test_list = [{"key": "value"}]
        
        mock_post = MagicMock(side_effect=requests.exceptions.RequestException())
        with patch.object(self.test_session.http_client, 'post', mock_post):
            with patch('time.sleep') as mock_sleep:
                 self.test_session.run()

        mock_post.assert_called_with("api/experiment/test", data={"key": "value"}, headers={"Authorization": "Bearer test_token"})
        self.assertEqual(mock_sleep.call_count, 2)  # Verify that it retried 3 times

    def test_run_key_error_no_retry(self):
        self.test_session.added = True
        self.test_session.test_list = [{"key": "value"}]
        
        mock_post = MagicMock(side_effect=KeyError("Invalid key"))
        with patch.object(self.test_session.http_client, 'post', mock_post), \
            mock.patch('builtins.print') as mock_print:
            with self.assertRaises(SystemExit) as cm:
                self.test_session.run()

            mock_print.assert_called_with("Key error occurred: 'Invalid key'")

    def test_run_value_error_no_retry(self):
        self.test_session.added = True
        self.test_session.test_list = [{"key": "value"}]
        
        mock_post = MagicMock(side_effect=ValueError("Invalid value"))
        with patch.object(self.test_session.http_client, 'post', mock_post):
            with self.assertRaises(SystemExit) as cm, \
                mock.patch('builtins.print') as mock_print:
                self.test_session.run()

        mock_print.assert_called_with("Value error occurred: Invalid value")

    def test_run_unexpected_error_no_retry(self):
        self.test_session.added = True
        self.test_session.test_list = [{"key": "value"}]
        
        mock_post = MagicMock(side_effect=Exception("Unexpected error"))
        with patch.object(self.test_session.http_client, 'post', mock_post):
            with self.assertRaises(SystemExit) as cm, \
                mock.patch('builtins.print') as mock_print:
                self.test_session.run()

        mock_print.assert_called_with("An unexpected error occurred: Unexpected error")

if __name__ == '__main__':
    unittest.main()




class TestSessionValidator:
    @staticmethod
    def validate_project_name(project_name: str) -> str:
        if not project_name:
            raise ValueError("Project name is required.")

        # Check if project_name contains any special characters except "_"
        allowed_chars = set("._-abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789 ")
        if not set(project_name).issubset(allowed_chars):
            raise ValueError("Project name should only contain alphanumeric characters and '_', '.'.")

        # Check if the project_name contains at least one alphabet character
        if not any(char.isalpha() for char in project_name):
            raise ValueError("Project name should contain at least one alphabet character.")

        # Additional validation logic specific to project_name, e.g., checking length or format
        return project_name

    @staticmethod
    def validate_run_name(run_name: str) -> str:
        if not run_name:
            raise ValueError("Run name is required.")

        # Check if run_name contains any special characters except "_"
        allowed_chars = set("._-abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789 ")
        if not set(run_name).issubset(allowed_chars):
            raise ValueError("Run name should only contain alphanumeric characters and '_', '.'.")

        # Check if the name contains at least one alphabet character
        if not any(char.isalpha() for char in run_name):
            raise ValueError("Run name should contain at least one alphabet character.")

        # Additional validation logic specific to name, e.g., checking length or format
        return run_name





from raga import *
import pandas as pd
import json
import datetime

def get_timestamp_x_hours_ago(hours):
    current_time = datetime.datetime.now()
    delta = datetime.timedelta(days=90, hours=hours)
    past_time = current_time - delta
    timestamp = int(past_time.timestamp())
    return timestamp

def convert_json_to_data_frame(json_file_path_model_1, json_file_path_model_2):
    test_data_frame = []
    with open(json_file_path_model_1, 'r') as json_file:
        # Load JSON data
        model_1 = json.load(json_file)
    with open(json_file_path_model_2, 'r') as json_file:
        # Load JSON data
        model_2 = json.load(json_file)

    # Create a dictionary to store the inputs and corresponding data points
    inputs_dict = {}
    hr = 1
    # Process model_1 data
    for item in model_1:
        inputs = item["inputs"]
        inputs_dict[tuple(inputs)] = item
    
    # Process model_2 data
    for item in model_2:
        inputs = item["inputs"]
        AnnotationsV1 = ImageDetectionObject()
        ROIVectorsM1 = ROIEmbedding()
        ImageVectorsM1 = ImageEmbedding()
        for detection in item["outputs"][0]["detections"]:
            AnnotationsV1.add(ObjectDetection(Id=0, ClassId=0, ClassName=detection['class'], Confidence=detection['confidence'], BBox= detection['bbox'], Format="xywh_normalized"))
            for roi_emb in detection['roi_embedding']:
                ROIVectorsM1.add(Embedding(roi_emb))
            attributes_dict = {}
            attributes = item.get("attributes", {})
            for key, value in attributes.items():
                attributes_dict[key] = StringElement(value)
            image_embeddings = item.get("image_embedding", {})
            for value in image_embeddings:
                ImageVectorsM1.add(Embedding(value))

        merged_item = inputs_dict.get(tuple(inputs), {})
        AnnotationsV2 = ImageDetectionObject()
        ROIVectorsM2 = ROIEmbedding()
        ImageVectorsM2 = ImageEmbedding()
        for detection in merged_item["outputs"][0]["detections"]:
            AnnotationsV2.add(ObjectDetection(Id=0, ClassId=0, ClassName=detection['class'], Confidence=detection['confidence'], BBox= detection['bbox'], Format="xywh_normalized"))
            for roi_emb in detection['roi_embedding']:
                ROIVectorsM2.add(Embedding(roi_emb))
        
        image_embeddings = merged_item.get("image_embedding", {})
        for value in image_embeddings:
            ImageVectorsM2.add(Embedding(value))

        data_point = {
            'ImageId': StringElement(item["inputs"][0]),
            'TimeOfCapture': TimeStampElement(get_timestamp_x_hours_ago(hr)),
            'SourceLink': StringElement(item["inputs"][0]),
            'AnnotationsV1': AnnotationsV1,
            'ROIVectorsM1': ROIVectorsM1,
            'ImageVectorsM1': ImageVectorsM1,
            'AnnotationsV2': AnnotationsV2,
            'ROIVectorsM2': ROIVectorsM2,
            'ImageVectorsM2': ImageVectorsM2,
        }

        merged_dict = {**data_point, **attributes_dict}
        test_data_frame.append(merged_dict)
        hr+=1

    return test_data_frame



#Convert JSON dataset to pandas Data Frame
pd_data_frame = pd.DataFrame(convert_json_to_data_frame("test-dataset-modelA.json", "test-dataset-modelB.json"))

pd_data_frame.to_pickle("TestingDataFrame.pkl")


# create schema object of RagaSchema instance
# schema = RagaSchema()
# schema.add("ImageId", PredictionSchemaElement(), pd_data_frame)
# schema.add("TimeOfCapture", TimeOfCaptureSchemaElement(), pd_data_frame)
# schema.add("SourceLink", FeatureSchemaElement(), pd_data_frame)
# schema.add("Reflection", AttributeSchemaElement(), pd_data_frame)
# schema.add("Overlap", AttributeSchemaElement(), pd_data_frame)
# schema.add("CameraAngle", AttributeSchemaElement(), pd_data_frame)
# schema.add("AnnotationsV1", InferenceSchemaElement(model="modelA-column-name-test-1"), pd_data_frame)
# schema.add("ImageVectorsM1", ImageEmbeddingSchemaElement(model="modelA-column-name-test-1", ref_col_name=""), pd_data_frame)
# schema.add("ROIVectorsM1", RoiEmbeddingSchemaElement(model="modelA-column-name-test-1", ref_col_name=""), pd_data_frame)
# schema.add("AnnotationsV2", InferenceSchemaElement(model="modelB-column-name-test-1"), pd_data_frame)
# schema.add("ImageVectorsM2", ImageEmbeddingSchemaElement(model="modelB-column-name-test-1", ref_col_name=""), pd_data_frame)
# schema.add("ROIVectorsM2", RoiEmbeddingSchemaElement(model="modelB-column-name-test-1", ref_col_name=""), pd_data_frame)

# #create test_session object of TestSession instance
# test_session = TestSession(project_name="testingProject",run_name= "test-exp-jun-5")

# #create test_ds object of Dataset instance
# test_ds = Dataset(test_session=test_session, name="test-dataset-jun-5")

# #load schema and pandas data frame
# test_ds.load(data=pd_data_frame, schema=schema)


# #Params for unlabelled AB Model Testing
# testName = StringElement("test-jun-5")
# modelA = StringElement("modelA-column-name-test-1")
# modelB = StringElement("modelB-column-name-test-1")
# type = ModelABTestTypeElement("unlabelled")
# aggregation_level = AggregationLevelElement()
# aggregation_level.add(StringElement("Reflection"))
# aggregation_level.add(StringElement("Overlap"))
# aggregation_level.add(StringElement("CameraAngle"))
# rules = ModelABTestRules()
# rules.add(metric = StringElement("difference_percentage"), IoU = FloatElement(0.5), _class = StringElement("ALL"), threshold = FloatElement(0.2))
# rules.add(metric = StringElement("difference_count"), IoU = FloatElement(0.5), _class = StringElement("canned_food"), threshold = FloatElement(0.5))

# #create payload for model ab testing
# model_comparison_check = model_ab_test(test_ds, testName=testName, modelA = modelA , modelB = modelB , type = type, aggregation_level = aggregation_level, rules = rules)

# #add payload into test_session object
# test_session.add(model_comparison_check)

# #run added ab test model payload
# test_session.run()


from typing import Optional
import pandas as pd
from raga.dataset_creds import DatasetCreds

class DatasetValidator:
    @staticmethod
    def validate_test_df(test_df: pd.DataFrame) -> pd.DataFrame:
        if not isinstance(test_df, pd.DataFrame):
            raise TypeError("Test dataframe must be of type pd.DataFrame.")
        # Additional validation logic specific to test_df, e.g., checking required columns or shape
        return test_df

    @staticmethod
    def validate_name(name: str) -> str:
        if not name:
            raise ValueError("Name is required.")

        # Check if name contains any special characters except "_"
        allowed_chars = set("._-abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789 ")
        if not set(name).issubset(allowed_chars):
            raise ValueError("Name should only contain alphanumeric characters and '_', '.'.")

        # Check if the name contains at least one alphabet character
        if not any(char.isalpha() for char in name):
            raise ValueError("Name should contain at least one alphabet character.")

        # Additional validation logic specific to name, e.g., checking length or format
        return name

    @staticmethod
    def validate_creds(creds: Optional[DatasetCreds] = None) -> Optional[DatasetCreds]:
        if creds is not None and not isinstance(creds, DatasetCreds):
            raise TypeError("DatasetCreds must be an instance of the DatasetCreds class.")
        # Additional validation logic specific to creds, e.g., checking authentication
        return creds



import os
import configparser
from raga import RAGA_CONFIG_FILE
import sys

DEFAULT_CONFIG_VALUES = {
    "api_host": "https://example.com",
    "raga_access_key_id": "your-access-key",
    "raga_secret_access_key": "your-secret-key"
}

def read_raga_config():
    config_file_path = os.path.expanduser(os.path.join("~", RAGA_CONFIG_FILE))

    if not os.path.isfile(config_file_path):
        create_default_config(config_file_path)
        print(f"A default config file has been created. Please update the credentials in the config file. You can update using this command `sudo vim {config_file_path}`")
        sys.exit(0)

    config = configparser.ConfigParser()
    try:
        config.read(config_file_path)
    except configparser.Error as e:
        raise ValueError(f"Invalid config file format: {str(e)}")

    validate_default_section(config)

    config_data = {}
    for section_name in config.sections():
        config_data[section_name] = dict(config.items(section_name))

    return config_data

def create_default_config(config_file_path):
    config = configparser.ConfigParser()
    config.add_section("default")

    for option, value in DEFAULT_CONFIG_VALUES.items():
        config.set("default", option, value)

    os.makedirs(os.path.dirname(config_file_path), exist_ok=True)

    with open(config_file_path, "w") as config_file:
        config.write(config_file)

def validate_default_section(config):
    default_section = config["default"]

    for option, default_value in DEFAULT_CONFIG_VALUES.items():
        if option not in default_section or default_section[option] == default_value:
            raise ValueError(f"Please update the value of '{option}' in the [default] section of the config file.")

def get_config_value(config_data, section, option):
    if section in config_data:
        section_data = config_data[section]
        if option in section_data:
            return section_data[option]
        else:
            raise KeyError(f"Option '{option}' not found in section '{section}'.")
    else:
        raise KeyError(f"Section '{section}' not found in config data.")



from raga import *
import pandas as pd

pd_data_frame = pd.read_pickle('TestingDataFrame.pkl')

#create schema object of RagaSchema instance
schema = RagaSchema()
schema.add("ImageId", PredictionSchemaElement(), pd_data_frame)
schema.add("TimeOfCapture", TimeOfCaptureSchemaElement(), pd_data_frame)
schema.add("SourceLink", FeatureSchemaElement(), pd_data_frame)
schema.add("Reflection", AttributeSchemaElement(), pd_data_frame)
schema.add("Overlap", AttributeSchemaElement(), pd_data_frame)
schema.add("CameraAngle", AttributeSchemaElement(), pd_data_frame)
schema.add("AnnotationsV1", InferenceSchemaElement(model="Testing-modelA"), pd_data_frame)
schema.add("ImageVectorsM1", ImageEmbeddingSchemaElement(model="Testing-modelA", ref_col_name=""), pd_data_frame)
schema.add("ROIVectorsM1", RoiEmbeddingSchemaElement(model="Testing-modelA", ref_col_name=""), pd_data_frame)
schema.add("AnnotationsV2", InferenceSchemaElement(model="Testing-modelB"), pd_data_frame)
schema.add("ImageVectorsM2", ImageEmbeddingSchemaElement(model="Testing-modelB", ref_col_name=""), pd_data_frame)
schema.add("ROIVectorsM2", RoiEmbeddingSchemaElement(model="Testing-modelB", ref_col_name=""), pd_data_frame)

#create test_session object of TestSession instance
test_session = TestSession(project_name="testingProject",run_name= "test-exp-jun-new-4")

#create test_ds object of Dataset instance
test_ds = Dataset(test_session=test_session, name="test-dataset-jun-new-1")

#load schema and pandas data frame
test_ds.load(pd_data_frame, schema)


#Params for unlabelled AB Model Testing
# testName = StringElement("test-unlabelled-1")
# modelA = StringElement("Testing-modelA")
# modelB = StringElement("Testing-modelB")
# type = ModelABTestTypeElement("labelled")
# aggregation_level = AggregationLevelElement()
# aggregation_level.add(StringElement("Reflection"))
# aggregation_level.add(StringElement("Overlap"))
# aggregation_level.add(StringElement("CameraAngle"))
# rules = ModelABTestRules()
# rules.add(metric = StringElement("difference_percentage"), IoU = FloatElement(0.5), _class = StringElement("ALL"), threshold = FloatElement(0.2))
# rules.add(metric = StringElement("difference_count"), IoU = FloatElement(0.5), _class = StringElement("canned_food"), threshold = FloatElement(0.5))

# #create payload for model ab testing
# model_comparison_check = model_ab_test(test_ds, testName=testName, modelA = modelA , modelB = modelB , type = type, aggregation_level = aggregation_level, rules = rules)

# #add payload into test_session object
# test_session.add(model_comparison_check)

# #run added ab test model payload
# test_session.run()



import logging
import os
import pandas as pd
import requests
import zipfile


logger = logging.getLogger(__name__)


class FileUploadError(Exception):
    pass

def data_frame_extractor(test_df: pd.DataFrame):
    dr = []
    for index, row in test_df.iterrows():
        df = {}
        for column_name in test_df.columns:
            element = row[column_name]
            df[column_name] = element.get()
        dr.append(df)
    return pd.DataFrame(dr)

def make_arg(model_name, inference_col_name, embedding_col_name):
    return [
        {"customerColumnName": inference_col_name, "type": "inference", "model": model_name, "ref_col_name": ""},
        {"customerColumnName": embedding_col_name, "type": "imageEmbedding", "model": model_name, "ref_col_name": ""}
    ]


def upload_file(pre_signed_url, file_path, success_callback=None, failure_callback=None):
    """
    Uploads a file to the server using a pre-signed URL.

    Args:
        pre_signed_url (str): The pre-signed URL for uploading the file.
        file_path (str): The path of the file to be uploaded.
        success_callback (Optional[function]): The callback function to be called on successful upload.
        failure_callback (Optional[function]): The callback function to be called on upload failure.

    Raises:
        ValueError: If the pre-signed URL or file path is missing.
        FileUploadError: If there is an error uploading the file.
    """
    if not pre_signed_url:
        raise ValueError("Pre-signed URL is required.")
    if not file_path:
        raise ValueError("File path is required.")
    if not os.path.isfile(file_path):
        raise ValueError(f"Invalid file path: {file_path}.")
    logger.debug(f"UPLOADING {file_path}")
    try:
        with open(file_path, "rb") as file:
            response = requests.put(pre_signed_url, data=file, headers={"Content-Type": "application/zip"})
            response.raise_for_status()  # Raise an exception for HTTP errors
            if response.status_code == 200:
                if success_callback:
                    success_callback()
                return True
            else:
                if failure_callback:
                    failure_callback(response.status_code)
                raise FileUploadError(f"File upload failed with status code: {response.status_code}")
    except requests.RequestException as e:
        if failure_callback:
            failure_callback(str(e))
        raise FileUploadError(f"Error uploading file: {e}")


def create_csv_and_zip_from_data_frame(data_frame:pd.DataFrame, csv_file, zip_file):
        """
        Creates a CSV file from the data frame and compresses it into a zip file.
        """
        # Validate the CSV file path
        if not csv_file or not isinstance(csv_file, str):
            raise ValueError(
                "Invalid CSV file path. Please provide a valid file path.")

        # Save the DataFrame as a CSV file
        data_frame.to_csv(csv_file, index=False)
        logger.debug("Data frame has been saved to CSV")

        # Create a zip file containing the CSV file
        with zipfile.ZipFile(zip_file, "w") as zip_file:
            zip_file.write(csv_file, os.path.basename(csv_file))

        logger.debug(f"CSV file has been zipped: {zip_file}")
        print(f"CSV file has been zipped: {zip_file}")


def delete_files(csv_file, zip_file):
    """
    Deletes the CSV and zip files associated with the dataset.
    """
    if os.path.exists(csv_file):
        os.remove(csv_file)
        logger.debug("CSV file deleted")
    else:
        logger.debug("CSV file not found")
        raise FileNotFoundError("CSV file not found.")

    if os.path.exists(zip_file):
        os.remove(zip_file)
        logger.debug("Zip file deleted")
    else:
        logger.debug("Zip file not found")
        raise FileNotFoundError("Zip file not found.")
    return True

def on_upload_success():
    """
    Callback function to be called on successful file upload.
    """
    logger.debug("File uploaded successfully")
    print("File uploaded successfully")

def on_upload_failed(error):
    """
    Callback function to be called on file upload failure.
    """
    logger.debug(f"ERROR: {error}")
    print("File upload failed")


import unittest
from raga import Dataset, StringElement, AggregationLevelElement, ModelABTestRules, ModelABTestTypeElement
from raga import model_ab_test, ab_test_validation, TestSession, FloatElement
from unittest import mock
from unittest.mock import MagicMock, patch, Mock, call

class TestABTestTestCase(unittest.TestCase):
    def setUp(self):
        self.test_session = TestSession("project_name", "run_name", u_test=True)
        self.test_session.project_id = "project_id"
        self.test_session.token = "test_token"
        self.test_session.api_host = "base_url"
        self.dataset_name = "my_dataset"
        self.test_session.experiment_id = "experiment_id"
        self.dataset_creds = None
        self.test_ds = Dataset(self.test_session, self.dataset_name, self.dataset_creds, u_test=True)
        self.test_ds.dataset_id = "12345"


        self.testName = StringElement("TestName")
        self.modelA = StringElement("modelA")
        self.modelB = StringElement("modelB")
        self.gt = StringElement("GT")
        self.type = ModelABTestTypeElement("unlabelled")
        self.filter = StringElement("unlabelled")
        self.aggregation_level = AggregationLevelElement()
        self.aggregation_level.add(StringElement("weather"))
        self.aggregation_level.add(StringElement("scene"))
        self.aggregation_level.add(StringElement("time_of_day"))
        self.rules = ModelABTestRules()
        self.rules.add(metric = StringElement("precision_diff"), IoU = FloatElement(0.5), _class = StringElement("all"), threshold = FloatElement(0.05))
        self.rules.add(metric = StringElement("‘difference_count’"), IoU = FloatElement(0.5), _class = StringElement("‘vehicle’"), threshold = FloatElement(0.05))

    def test_ab_test_validation(self):
        # Test valid inputs
        self.assertTrue(ab_test_validation(self.test_ds, self.testName, self.modelA, self.modelB,
                                            self.type, self.aggregation_level, self.rules))

        # Test invalid test_ds
        with self.assertRaises(AssertionError):
            ab_test_validation("invalid_test_ds", self.testName, self.modelA, self.modelB,
                                self.type, self.aggregation_level, self.rules)

        # Test missing testName
        with self.assertRaises(AssertionError):
            ab_test_validation(self.test_ds, None, self.modelA, self.modelB,
                                self.type, self.aggregation_level, self.rules)

        # Test missing modelA
        with self.assertRaises(AssertionError):
            ab_test_validation(self.test_ds, self.testName, None, self.modelB,
                                self.type, self.aggregation_level, self.rules)

        # Test missing modelB
        with self.assertRaises(AssertionError):
            ab_test_validation(self.test_ds, self.testName, self.modelA, None,
                                self.type, self.aggregation_level, self.rules)
            
        # Test missing type
        with self.assertRaises(AssertionError):
            ab_test_validation(self.test_ds, self.testName, self.modelA, self.modelB,
                                None, self.aggregation_level, self.rules)
            
        # Test missing aggregation_level
        with self.assertRaises(AssertionError):
            ab_test_validation(self.test_ds, self.testName, self.modelA, self.modelB,
                                self.type, None, self.rules)
            
        # Test missing rules
        with self.assertRaises(AssertionError):
            ab_test_validation(self.test_ds, self.testName, self.modelA, self.modelB,
                                self.type, self.aggregation_level, None)
        
        # Test type labelled
        self.type = ModelABTestTypeElement("labelled")
        self.gt = StringElement("")
        with self.assertRaises(AssertionError):
            ab_test_validation(test_ds=self.test_ds, testName=self.testName, modelA=self.modelA, modelB=self.modelB,
                                type=self.type, aggregation_level=self.aggregation_level, rules=self.rules, gt=self.gt)

         # Test type unlabelled
        self.type = ModelABTestTypeElement("unlabelled")
        self.gt = StringElement("GT")
        with self.assertRaises(ValueError):
            ab_test_validation(test_ds=self.test_ds, testName=self.testName, modelA=self.modelA, modelB=self.modelB,
                                type=self.type, aggregation_level=self.aggregation_level, rules=self.rules, gt=self.gt)

    def test_model_ab_test_labelled(self):
        # Test valid inputs
        self.type = ModelABTestTypeElement("labelled")
        self.gt = StringElement("GT")
        result = model_ab_test(test_ds=self.test_ds, testName=self.testName, modelA=self.modelA, modelB=self.modelB,
                                type=self.type, aggregation_level=self.aggregation_level, rules=self.rules, filter=self.filter, gt=self.gt)
        self.assertEqual(result["datasetId"], self.test_ds.dataset_id)
        self.assertEqual(result["experimentId"], self.test_ds.test_session.experiment_id)
        self.assertEqual(result["name"], self.testName.get())
        self.assertEqual(result["filter"], self.filter.get())
        self.assertEqual(result["modelA"], self.modelA.get())
        self.assertEqual(result["modelB"], self.modelB.get())
        self.assertEqual(result["type"], self.type.get())
        self.assertEqual(result["aggregationLevels"], self.aggregation_level.get())
        self.assertEqual(result["rules"], self.rules.get())
        self.assertEqual(result["gt"], self.gt.get())
        
        with unittest.mock.patch("raga._tests.ab_test_validation") as mock_validation:
            model_ab_test(test_ds=self.test_ds, testName=self.testName, modelA=self.modelA, modelB=self.modelB,
                                type=self.type, aggregation_level=self.aggregation_level, rules=self.rules, gt= self.gt)
            mock_validation.assert_called_with(self.test_ds,self.testName, self.modelA, self.modelB, self.type, self.aggregation_level, self.rules, self.gt)

    def test_model_ab_test_unlabelled(self):
        # Test valid inputs
        self.type = ModelABTestTypeElement("unlabelled")
        result = model_ab_test(test_ds=self.test_ds, testName=self.testName, modelA=self.modelA, modelB=self.modelB,
                                type=self.type, aggregation_level=self.aggregation_level, rules=self.rules, filter=self.filter)
        self.assertEqual(result["datasetId"], self.test_ds.dataset_id)
        self.assertEqual(result["experimentId"], self.test_ds.test_session.experiment_id)
        self.assertEqual(result["name"], self.testName.get())
        self.assertEqual(result["filter"], self.filter.get())
        self.assertEqual(result["modelA"], self.modelA.get())
        self.assertEqual(result["modelB"], self.modelB.get())
        self.assertEqual(result["type"], self.type.get())
        self.assertEqual(result["aggregationLevels"], self.aggregation_level.get())
        self.assertEqual(result["rules"], self.rules.get())

        with unittest.mock.patch("raga._tests.ab_test_validation") as mock_validation:
            model_ab_test(test_ds=self.test_ds, testName=self.testName, modelA=self.modelA, modelB=self.modelB,
                                type=self.type, aggregation_level=self.aggregation_level, rules=self.rules, gt= self.gt)
            mock_validation.assert_called_with(self.test_ds,self.testName, self.modelA, self.modelB, self.type, self.aggregation_level, self.rules, self.gt)
        


if __name__ == '__main__':
    unittest.main()